###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH.
# All Rights Reserved.
#
##############################################################################
"""Source code checker, to be used in unittests (original written by agroszer)

$Id: checker.py 5660 2025-09-02 12:49:03Z roger.ineichen $
"""

from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
from builtins import object

import re
import sys
import os
import os.path
import zlib
import string
import logging
import polib
import lxml.etree
import subprocess
try:
    from io import StringIO
    from io import BytesIO
    from codecs import open as fOpen
except ImportError:
    from io import StringIO
    from io import BytesIO
    from io import open as fOpen

try:
    unicode             # p01.checker.silence
    basestring          # p01.checker.silence
except Exception:
    unicode = str       # p01.checker.silence
    basestring = str    # p01.checker.silence

PY2 = sys.version_info[0] == 2
if PY2:
    text_type = unicode                   # p01.checker.silence
    string_types = (basestring, unicode)  # p01.checker.silence
else:
    text_type = str
    string_types = (str,)

import lxml.etree

from p01.checker.pyflakes import checkPYFlakes


###############################################################################
#
# checker classes
#
###############################################################################

class BaseChecker(object):
    """Checker base class"""

    error = None

    def log(self, idx, line=None, pos=None, error=None):
        if error is None:
            error = self.error
        return {
            'checker': self,
            'error': error,
            'idx': idx,
            'line': line,
            'pos': pos,
            }

    def check(self, basename, filename, content, lines):
        return


class TabChecker(BaseChecker):
    """Text tabulator checker"""

    error = 'tab found in file'

    def check(self, basename, filename, content, lines):
        for idx, line in enumerate(lines):
            if '\t' in line:
                yield self.log(idx, line)


VALIDCHARS = string.printable


def replaceUnicode(line):
    try:
        return line.encode('utf-8', 'replace')
    except UnicodeEncodeError:
        return line

class NonASCIIChecker(BaseChecker):
    """Non ascii text checker"""

    error = "non ASCII char found"

    def check(self, basename, filename, content, lines):
        for idx, line in enumerate(lines):
            for cidx, c in enumerate(line):
                if c not in VALIDCHARS and not 'p01.checker.silence' in line:
                    yield self.log(idx, replaceUnicode(line), cidx)
                    break


class BreakPointChecker(BaseChecker):
    """Debug break point checker"""

    error = 'debug break point found'
    parts = ['pdb.set_trace', 'from pub.dbgpclient import brk'] # p01.checker.silence

    def check(self, basename, filename, content, lines):
        for part in self.parts:
            for idx, line in enumerate(lines):
                if part in line \
                    and not (-1 < line.find('#') < line.find(part)) \
                    and not '# p01.checker.silence' in line:
                    line = '%s # p01.checker.silence' % line
                    yield self.log(idx, line)


class OpenInBrowserChecker(BaseChecker):
    """Open in browser method checker"""

    error = 'openInBrowser found'

    def check(self, basename, filename, content, lines):
        key = '.open' + 'InBrowser'
        for idx, line in enumerate(lines):
            if (key in line
                and not (-1 < line.find('#') < line.find(key))):
                yield self.log(idx, line)


class PYFlakesChecker(BaseChecker):
    """Pyflakes checker"""

    error = 'pyflakes warning'

    def fixcontent(self, lines):
        #pyflakes does not like CRLF linefeeds
        #and files ending with comments
        idx = len(lines)-1
        lastline = lines[idx].strip()
        while idx >= 1 and (lastline == '' or lastline.startswith('#')):
            del lines[idx]
            idx -= 1
            lastline = lines[idx].strip()

        content = '\n'.join(lines)
        return content

    def check(self, basename, filename, content, lines):
        if "##skip pyflakes##" in content:
            return
        content = self.fixcontent(lines)
        try:
            result = checkPYFlakes(content, filename)
        except Exception as e:
            result = "Fatal PyFlakes exception: %s" % e

        if isinstance(result, string_types):
            #something fatal occurred
            self.error = result
            self.log(0)
        else:
            # there are messages
            for warning in result:
                if isinstance(warning, string_types):
                    yield self.log(warning, 0)
                elif ('undefined name' in warning.message
                    and not 'unable to detect undefined names' in warning.message):
                    self.error = warning.message % warning.message_args
                    line = lines[warning.lineno-1]
                    if isinstance(line, string_types) and not \
                        'p01.checker.silence' in line:
                        yield self.log(warning.lineno-1, line)


class ConsoleLogChecker(BaseChecker):
    """Javascript console log checker"""

    error = 'javascript console.log found'

    def check(self, basename, filename, content, lines):
        for idx, line in enumerate(lines):
            if 'console.log' in line \
                and not (-1 < line.find('//') < line.find('console.log')) \
                and not 'p01.checker.silence' in line:
                yield self.log(idx, line)


class AlertChecker(BaseChecker):
    """Javascript alert checker"""

    error = 'javascript alert found'

    def check(self, basename, filename, content, lines):
        for idx, line in enumerate(lines):
            if 'alert(' in line \
                and not (-1 < line.find('//') < line.find('alert(')) \
                and not 'p01.checker.silence' in line:
                yield self.log(idx, line)


class CSSLogger(object):
    # this is a fake logger that redirects the actual logging calls to us

    errors = []

    def __init__(self):
        self.errors = []

    def noop(self, *args, **kw):
        pass

    debug = noop
    info = noop
    setLevel = noop
    getEffectiveLevel = noop
    addHandler = noop
    removeHandler = noop

    def error(self, msg):
        try:
            error = str(msg)
        except UnicodeEncodeError:
            error = msg.encode('ascii', 'replace')
        error = error.strip()
        # can't add much help, all info is encoded in msg
        self.errors.append((0, None, None, error))

    warn = error
    critical = error
    fatal = error


class CSSChecker(BaseChecker):
    """CSS checker based on cssutils with lazy import

    NOTE: you must inlcude cssutils extras_require in your buildout with:

    p01.checker [csstuils]

    for include the cssutils dependencies. Otherwise we don't depend on
    cssutils
    """

    error = 'CSS'

    def check(self, basename, filename, content, lines):
        logger = CSSLogger()
        import cssutils.parse
        parser = cssutils.parse.CSSParser(log=logger, loglevel=logging.INFO,
            validate=True)
        parser.parseString(content)
        for idx, line, pos, error in logger.errors:
            yield self.log(idx, line, pos, error)


CSS_LINT_ALL_RULES = [
    'adjoining-classes',
    'box-model',
    'box-sizing',
    'compatible-vendor-prefixes',
    'display-property-grouping',
    'duplicate-background-images',
    'duplicate-properties',
    'empty-rules',
    'errors',
    'fallback-colors',
    'floats',
    'font-faces',
    'font-sizes',
    'gradients',
    'ids',
    'import',
    'important',
    'known-properties',
    'outline-none',
    'overqualified-elements',
    'qualified-headings',
    'regex-selectors',
    'rules-count',
    'shorthand',
    'star-property-hack',
    'text-indent',
    'underscore-property-hack',
    'unique-headings',
    'universal-selector',
    'unqualified-attributes',
    'vendor-prefix',
    'zero-units',
    ]

CSS_LINT_ERROR_RULES = [
    'empty-rules',
    'errors',
    'known-properties',
]


class CSSLintChecker(BaseChecker):
    """CSSLint checker based on rhino and css lint.

    See: https://github.com/stubbornella/csslint/wiki/Command-line-interface

    The css lint checker can get used with the following command line options:

    ge: csslint [options]* [file|dir]*

    Global Options
      --help                    Displays this information.
      --format=<format>         Indicate which format to use for output.
      --list-rules              Outputs all of the rules available.
      --quiet                   Only output when errors are present.
      --errors=<rule[,rule]+>   Indicate which rules to include as errors.
      --warnings=<rule[,rule]+> Indicate which rules to include as warnings.
      --ignore=<rule,[,rule]+>  Indicate which rules to ignore completely.
      --version                 Outputs the current version number.

    The output format from csslint with the --format=checkstyle-xml looks like:

    <?xml version="1.0" encoding="utf-8"?>
    <checkstyle>
      <file name="...\\p01.checker\\src\\p01\\checker\\sample\\bad.css">
        <error
            line="9"
            column="1"
            severity="warning"
            message="Element (div.bad_width) is overqualified, just use
                     .bad_width without element name."
            source="net.csslint.Disallowoverqualifiedelements"/>
        <error
            line="13"
            column="1"
            severity="warning"
            message="Element (div.missing_coma) is overqualified, just use
                     .missing_coma without element name."
            source="net.csslint.Disallowoverqualifiedelements"/>
        <error
            line="14"
            column="5"
            severity="warning"
            message="Expected end of value but found 'height'."
            source="net.csslint.Requireuseofknownproperties"/>
        <error
            line="15"
            column="11"
            severity="error"
            message="Expected RBRACE at line 15, col 11."
            source="net.csslint.ParsingErrors"/>
        <error
            line="19"
            column="1"
            severity="warning" message="Element (div.bad_value) is
                      overqualified, just use .bad_value without element name."
            source="net.csslint.Disallowoverqualifiedelements"/>
        <error
            line="20"
            column="5"
            severity="warning"
            message="Expected (&lt;absolute-size&gt; | &lt;relative-size&gt; |
                     &lt;length&gt; | &lt;percentage&gt; | inherit) but found
                     'inline-block'."
            source="net.csslint.Requireuseofknownproperties"/>
        <error
            line="26"
            column="1"
            severity="error"
            message="Expected LBRACE at line 26, col 1."
            source="net.csslint.ParsingErrors"/>
      </file>
    </checkstyle>

    available rules:

        adjoining-classes
          Don't use adjoining classes.

        box-model
          Don't use width or height when using padding or border.

        box-sizing
          The box-sizing properties isn't supported in IE6 and IE7.

        compatible-vendor-prefixes
          Include all compatible vendor prefixes to reach a wider range of users.

        display-property-grouping
          Certain properties shouldn't be used with certain display property values.

        duplicate-background-images
          Every background-image should be unique. Use a common class for e.g. sprites.

        duplicate-properties
          Duplicate properties must appear one after the other.

        empty-rules
          Rules without any properties specified should be removed.

        errors
          This rule looks for recoverable syntax errors.

        fallback-colors
          For older browsers that don't support RGBA, HSL, or HSLA, provide a fallback color.

        floats
          This rule tests if the float property is used too many times

        font-faces
          Too many different web fonts in the same stylesheet.

        font-sizes
          Checks the number of font-size declarations.

        gradients
          When using a vendor-prefixed gradient, make sure to use them all.

        ids
          Selectors should not contain IDs.

        import
          Don't use @import, use <link> instead.

        important
          Be careful when using !important declaration

        known-properties
          Properties should be known (listed in CSS3 specification) or be a vendor-prefixed property.

        outline-none
          Use of outline: none or outline: 0 should be limited to :focus rules.

        overqualified-elements
          Don't use classes or IDs with elements (a.foo or a#foo).

        qualified-headings
          Headings should not be qualified (namespaced).

        regex-selectors
          Selectors that look like regular expressions are slow and should be avoided.

        rules-count
          Track how many rules there are.

        shorthand
          Use shorthand properties where possible.

        star-property-hack
          Checks for the star property hack (targets IE6/7)

        text-indent
          Checks for text indent less than -99px

        underscore-property-hack
          Checks for the underscore property hack (targets IE6)

        unique-headings
          Headings should be defined only once.

        universal-selector
          The universal selector (*) is known to be slow.

        unqualified-attributes
          Unqualified attribute selectors are known to be slow.

        vendor-prefix
          When using a vendor-prefixed property, make sure to include the standard one.

        zero-units
          You don't need to specify units when a value is 0.

    """

    _rules = []
    error = 'CSSLint'

    def __init__(self, rules=None):
        # indicate which rules to include in check
        self._rules = []
        if rules is None:
            rules = CSS_LINT_ALL_RULES
        self.rules = CSS_LINT_ERROR_RULES

    @property
    def rules(self):
        return self._rules

    @rules.setter
    def rules(self, values):
        self._rules = values
        # remove given rules from ignore rules
        ignore = list(CSS_LINT_ALL_RULES)
        for rule in values:
            # remove rule from ignore
            ignore.remove(rule)
        self.ignore = ignore

    def check(self, basename, filename, content, lines):
        """Process css lint command

        java -jar js.jar csslint-rhino.js --format=lint-xml path/to/file.css

        """
        here = os.path.dirname(__file__)
        lintDir = os.path.join(here, 'csslint')
        os.chdir(lintDir)

        # setup java options and friends
        JAVA_HOME = os.environ.get('JAVA_HOME')
        if JAVA_HOME is not None and os.path.exists(JAVA_HOME):
            JAVA_BIN = os.path.join(JAVA_HOME, 'bin', 'java')
        else:
            JAVA_BIN = 'java'
        cmd = [
            JAVA_BIN,
            "-jar",
            "js.jar",
            "csslint-rhino.js",
            "--format=checkstyle-xml",
        ]
        if self.rules is not None:
            if isinstance(self.rules, (list, tuple)):
                rules = ','.join(self.rules)
            else:
                rules = self.rules
            cmd += ['--errors=%s' % rules]
        if self.ignore is not None:
            if isinstance(self.ignore, (list, tuple)):
                ignore = ','.join(self.ignore)
            else:
                ignore = self.ignore
            cmd += ['--ignore=%s' % ignore]
        # append filename
        cmd += [filename]

        try:
            pro = subprocess.Popen(cmd, cwd=lintDir, stdin=subprocess.PIPE,
                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            stdout, stderr = pro.communicate()
            code = pro.returncode
            if stderr:
                code = 1
        except Exception as e:
            raise Exception("CSSLint subprocess error: %s" % e)

        # setup lines for report them later
        idx = 0
        code = {}
        #f = open(filename, 'rb')
        # f = open(filename, 'rb')
        # for line in f:
        #     code[idx] = line
        #     idx += 1
        # f.close()
        with fOpen(filename, 'r', encoding='utf-8') as f:
            for line in f:
                code[idx] = line
                idx += 1

        # parse checkstyle xml
        etree = lxml.etree.fromstring(stdout,
            parser=lxml.etree.XMLParser(resolve_entities=False))

        for issue in etree.xpath('//checkstyle/file/error'):
            idx = issue.get('line')
            try:
                idx = int(idx) -1
                line = code.get(idx)
            except (TypeError, ValueError):
                idx = 0
                line = None
            error = issue.get('message')
            yield self.log(idx, line, None, error)


class ImageChecker(BaseChecker):
    """Image compression checker"""

    error = 'image bloat found'

    def check(self, basename, filename, content, lines):
        
        if b"ns.adobe.com" in content:
            yield self.log(0, "Adobe Photoshop bloat found")
            return

        if b"<rdf:RDF" in content:
            yield self.log(0, "RDF bloat found")
            return

        if len(content) < 500:
            return

        compressed = zlib.compress(content)
        #compressed = zlib.compress(content)
        ratio = len(compressed) / float(len(content)-200)
        #200= circa static header length

        if ratio < 0.8:
            line = "Some other bloat found, compression ratio: %s" %ratio
            yield self.log(0, line)


class JPGChecker(ImageChecker):
    """JPG image compression checker"""


class PNGChecker(ImageChecker):
    """PNG image compression checker"""


class POChecker(BaseChecker):
    """PO fuzzy counter checker"""

    error = 'fuzzy/untranslated found'

    def check(self, basename, filename, content, lines):
        pos = polib.pofile(filename)
        untrans = pos.untranslated_entries()
        fuzzy = pos.fuzzy_entries()

        if len(untrans) > 0:
            error = 'untranslated found'
            line = "%s items" % len(untrans)
            yield self.log(0, line, None, error)

        if len(fuzzy) > 0:
            error = 'fuzzy found'
            line = "%s items" % len(fuzzy)
            yield self.log(0, line, None, error)


class BadI18nDomainChecker(BaseChecker):
    """Bad i18n domain checker"""

    error = 'bad i18n domain found'

    def __init__(self, pattern, pathPart, domains=()):
        self.pattern = pattern
        self.pathPart = pathPart
        self.domains = domains

    def check(self, basename, filename, content, lines):
        # make it linux-ish
        error = self.error
        filename = filename.replace('\\', '/')

        if self.pathPart in filename:
            for idx, line in enumerate(lines):
                if 'i18n:domain=""' in line:
                    error = 'empty i18n:domain="" found'
                    yield self.log(idx, line, None, error)
                if 'i18n_domain=""' in line:
                    error = 'empty i18n_domain="" found'
                    yield self.log(idx, line, None, error)
                if self.pattern.search(content):
                    for fdomain in self.pattern.findall(line):
                        if fdomain == '' or fdomain not in self.domains:
                            error = 'bad i18n domain found'
                            yield self.log(idx, line, None, error)


class ZCMLBadI18nDomainChecker(BadI18nDomainChecker):
    """Bad zcml i18n domain checker"""

    def __init__(self, pathPart, domains=()):
        pattern = re.compile(r'domain="(\w+)"')
        super(ZCMLBadI18nDomainChecker, self).__init__(pattern, pathPart,
            domains)


class PTBadI18nDomainChecker(BadI18nDomainChecker):
    """Bad pt i18n domain checker"""

    def __init__(self, pathPart, domains=()):
        pattern = re.compile(r'i18n:domain="(\w+)"')
        super(PTBadI18nDomainChecker, self).__init__(pattern, pathPart, domains)


HTML_WRAPPER = """
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:tal="http://xml.zope.org/namespaces/tal"
      xmlns:metal="http://xml.zope.org/namespaces/metal"
      xmlns:i18n="http://xml.zope.org/namespaces/i18n">
  %s
</html>

"""


class PTMissingDomainChecker(BaseChecker):
    """Missing i18n:domain checker"""

    def reprElement(self, ele):
        res = lxml.etree.tostring(ele)
        res = res.replace(b' xmlns="http://www.w3.org/1999/xhtml"', b'')
        res = res.replace(b' xmlns:tal="http://xml.zope.org/namespaces/tal"', b'')
        res = res.replace(b' xmlns:i18n="http://xml.zope.org/namespaces/i18n"', b'')
        res = res.replace(b' xmlns:metal="http://xml.zope.org/namespaces/metal"', b'')
        return res

    def checkDomain(self, ele):
        dKey = '{http://xml.zope.org/namespaces/i18n}domain'
        while ele is not None:
            if ele.attrib.get(dKey) is not None:
                return True
            ele = ele.getparent()
        return False

    def check(self, basename, filename, content, lines):
        if not 'i18n:translate' in content:
            # no translation
            return

        # checks for i18n:translate and find i18n:domain in parent elements
        # if no domain get found report line and element content
        lineCorrector = 0
        if not (content.startswith('<html') or \
                content.startswith('<!DOCTYPE') or \
                content.startswith('<!doctype') or \
                content.startswith('<?xml')):
            content = HTML_WRAPPER % content
            lineCorrector = 6
        sc = BytesIO(content.encode('utf-8'))
        try:
            for event, element in lxml.etree.iterparse(sc):
                # check if we have an i18n:translate without a i18n:domain
                # parent
                if '{http://xml.zope.org/namespaces/i18n}translate' in element.attrib:
                    # find i18n:domain in parent elements
                    if not self.checkDomain(element):
                        idx = element.sourceline - lineCorrector
                        line = self.reprElement(element)
                        error = "needs i18n:domain"
                        yield self.log(idx, line, None, error)
        except lxml.etree.XMLSyntaxError as e:
            # we do not validate XML in this check, see PTXHTMLChecker
            pass

class PTXHTMLChecker(BaseChecker):
    """Chameleon template checker

    Chameleon defines the following namespaces by default:

    xmlns:tal="http://xml.zope.org/namespaces/tal"
    xmlns:metal="http://xml.zope.org/namespaces/metal"
    xmlns:i18n="http://xml.zope.org/namespaces/i18n"

    This means the document does not have to define them. We will wrap the
    content with this namespaces if the document does not start with <html
    or <!DOCTYPE.

    """
    allowedErrorMassageStarts = [
        # umlaut
        "Entity 'Auml' not defined",
        "Entity 'auml' not defined",
        "Entity 'Ouml' not defined",
        "Entity 'ouml' not defined",
        "Entity 'Uuml' not defined",
        "Entity 'uuml' not defined",
        # other often used entities
        "Entity 'nbsp' not defined",
        "Entity 'ndash' not defined",
        "Entity 'raquo' not defined",
        "Entity 'times' not defined",
        # namespace prefixes
        'Namespace prefix i18n for ',
        'Namespace prefix i18n on ',
        'Namespace prefix metal for ',
        'Namespace prefix metal on ',
        'Namespace prefix tal for ',
        'Namespace prefix tal on ',
    ]

    def validateError(self, e):
        for allowed in self.allowedErrorMassageStarts:
           if e.message.startswith(allowed):
                return False
        return True

    def formatError(self, e, lineCorrector):
        eLine = e.line
        msg = e.message
        if lineCorrector:
            # adjust error line
            eLine = e.line - lineCorrector
            # probably there is a line reference in our error, adjust them
            number = re.search("line ([0-9]+)", msg)
            if number:
                orgNum = int(number.group(1))
                newNum = orgNum - lineCorrector
                orgStr = 'line %s' % orgNum
                newStr = 'line %s' % newNum
                msg = msg.replace(orgStr, newStr)
        return '%s, line: %s, block: %s' % (msg, eLine, e.column)

    def check(self, basename, filename, content, lines, enconding="utf-8"):
        error = None
        lineCorrector = 0

        if not (content.startswith('<html') or \
                content.startswith('<!DOCTYPE') or \
                content.startswith('<!doctype') or \
                content.startswith('<?xml')):
            content = HTML_WRAPPER % content
            lineCorrector = 6
        # sc = StringIO(content)
        try:
            p = lxml.etree.XMLParser()
            if isinstance(content, string_types):
                content = content.encode('utf-8')
            else:
                content = bytes(content)
            ignored = lxml.etree.XML(content, p)
        except lxml.etree.XMLSyntaxError as e:
            msg = '\n'.join(
                [self.formatError(e, lineCorrector)
                 for e in p.error_log
                 if self.validateError(e)
                 ])
            if msg:
                error = msg

        if isinstance(error, string_types):
            # something fatal occurred, use error as line
            yield self.log(None, error, None, 'fatal error')



##############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH.
# All Rights Reserved.
#
##############################################################################
"""Sample module supporting p01.checker.silence

$Id: silence.py 5170 2025-03-06 00:12:58Z felipe.souza $
"""

from __future__ import absolute_import
from builtins import object
class Foo(object):
    """A sample class forcing pyflake troubles"""

    def undefined_name(self):
        foo = False
        return bar

    def non_ascii_silence(self):
        return u'ö' # p01.checker.silence

    def non_ascii(self):
        return u'ö'



##############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH.
# All Rights Reserved.
#
##############################################################################
"""Sourcecode checker, to be used in unittests (by agroszer)

$Id: __init__.py 5622 2025-08-26 11:37:23Z roger.ineichen $
"""

from __future__ import absolute_import
from __future__ import print_function

from builtins import str
from builtins import object
import os.path
import sys

try:
    from codecs import open as fOpen
except ImportError:
    from io import open as fOpen

from p01.checker.checker import AlertChecker
from p01.checker.checker import BadI18nDomainChecker
from p01.checker.checker import BreakPointChecker
from p01.checker.checker import CSSChecker
from p01.checker.checker import CSSLintChecker
from p01.checker.checker import ConsoleLogChecker
from p01.checker.checker import JPGChecker
from p01.checker.checker import NonASCIIChecker
from p01.checker.checker import OpenInBrowserChecker
from p01.checker.checker import PNGChecker
from p01.checker.checker import POChecker
from p01.checker.checker import PTBadI18nDomainChecker
from p01.checker.checker import PTMissingDomainChecker
from p01.checker.checker import PTXHTMLChecker
from p01.checker.checker import PYFlakesChecker
from p01.checker.checker import TabChecker
from p01.checker.checker import ZCMLBadI18nDomainChecker


PY2 = sys.version_info[0] == 2
if PY2:
    string_types = (str, unicode)  # p01.checker.silence
else:
    string_types = (str,)


###############################################################################
#
# p01.checker.silence
#
# p01.checker.silence is not a variable, it's just a hint that you can
# silence output given from Checker classes. Just add the following
# comment behind a line of code if you like to
# skip checker alerts:
#
# import pdb;pdb.set_trace() # p01.checker.silence
#
# for HTML, javascript and page templates, you can use the phrase
# p01.checker.silence in the same line as the error message will get
# reported e.g.
#
# <!-- p01.checker.silence -->
#
# or
#
# // p01.checker.silence
#


###############################################################################
#
# p01.checker.ignore
#
# This line somewhere in a file will force to ignore checking the file at all
#


PY_CHECKERS = {
    'TabChecker': TabChecker(),
    'NonASCIIChecker': NonASCIIChecker(),
    'BreakPointChecker': BreakPointChecker(),
    'OpenInBrowserChecker': OpenInBrowserChecker(),
    'PYFlakesChecker': PYFlakesChecker(),
}
PT_CHECKERS = {
    'TabChecker': TabChecker(),
    'NonASCIIChecker': NonASCIIChecker(),
    'ConsoleLogChecker': ConsoleLogChecker(),
    'AlertChecker': AlertChecker(),
    # see checker.txt for how to add a page template domain checker
    #PTBadI18nDomainChecker('/p01/', ('p01',)),
    'PTXHTMLChecker': PTXHTMLChecker(),
    'PTMissingDomainChecker': PTMissingDomainChecker(),
}
JS_CHECKERS = {
    'TabChecker': TabChecker(),
    'ConsoleLogChecker': ConsoleLogChecker(),
    'AlertChecker': AlertChecker(),
}
CSS_CHECKERS = {
    # CSSChecker does not really produce usefull output, every second style
    # get reported as wrong e.g. -moz-border-radius, zoom, -ms-box-sizing
    #CSSChecker(),
    'CSSLintChecker': CSSLintChecker(),
    'NonASCIIChecker': NonASCIIChecker(),
}
TXT_CHECKERS = {
    'TabChecker': TabChecker(),
    'BreakPointChecker': BreakPointChecker(),
    'OpenInBrowserChecker': OpenInBrowserChecker(),
}
PO_CHECKERS = {
    'POChecker': POChecker(),
}
JPG_CHECKERS = {
    'JPGChecker': JPGChecker(),
}
PNG_CHECKERS = {
    'PNGChecker': PNGChecker(),
}
ZCML_CHECKERS = {
    # see checker.txt for how to add a zcml domain checker
    #ZCMLBadI18nDomainChecker('/p01/', ('p01',)),
}

CHECKERS = {
    'css': CSS_CHECKERS,
    'html': PT_CHECKERS,
    'jpg': JPG_CHECKERS,
    'png': PNG_CHECKERS,
    'js': JS_CHECKERS,
    'po': PO_CHECKERS,
    'pt': PT_CHECKERS,
    'py': PY_CHECKERS,
    'txt': TXT_CHECKERS,
    'zcml': ZCML_CHECKERS,
}


def replaceUnicode(txt):
    if isinstance(txt, bytes):
        return txt.decode('utf-8', 'replace')
    return txt

def sortErrors(error):
    return error.get('idx', 0)

class Checker(object):
    """Checker implementation"""

    intend = '      '
    intro = False
    filename = None
    basename = None

    def __init__(self, checkers=CHECKERS, intend=None):
        self.checkers = checkers
        if intend is not None:
            self.intend = intend

    def addChecker(self, ext, checker):
        """Add custom checker for given file name extension

        This is usefull for simply add a checker with a custom setup
        """
        checkers = self.checkers.setdefault(ext, {})
        checkers[checker.__class__.__name__] = checker

    def removeChecker(self, ext, name):
        """Remove checker for given file name extension"""
        checkers = self.checkers.get(ext, {})
        if name in checkers:
            del checkers[name]

    def start(self, basename, filename):
        """Log file intro"""
        # always slash please
        filename = filename.replace('\\','/')
        filename = filename[len(basename)+1:]
        print(('-'* (len(filename))))
        print(filename)
        print(('-'* (len(filename))))

    def log(self, data):
        idx = data.get('idx')
        line = data.get('line')
        pos = data.get('pos')
        error = data.get('error')

        # print(line no and error msg)
        if not idx:
            idx = '0: '
        else:
            idx = str(idx+1)+': '
        print(("%s%s" % (idx, error)))

        if line:
            # calculate strip lenght
            l = len(line)
            line = line.strip()
            skip = l - len(line)
            # print(line)
            for l in line.splitlines():
                print(("%s%s" % (self.intend, replaceUnicode(l).strip())))
                # print("%s%s" % (self.intend, l.strip()))
            # optional mark position
            if pos is not None and pos -skip <= len(line):
                print(("%s%s^" % (self.intend, ' '*(pos -skip))))

    def summary(self, basename, filename, errors=[]):
        """Report errors per file"""
        if len(errors):
            self.start(basename, filename)
            for data in sorted(errors, key=sortErrors):
                # log data per checker
                self.log(data)

    def check(self, module, skipFileNames=[], skipCheckerFileNames={},
        skipFolderNames=[]):
        """Thread save checker method"""
        basename = os.path.dirname(module.__file__)
        for root, dirs, files in os.walk(basename, topdown=True):
            # keep the name order
            dirs.sort()
            files.sort()

            # remove skipped folders from walk list
            for dname in skipFolderNames:
                if dname in dirs:
                    dirs.remove(dname)

            for name in files:
                if not name in skipFileNames:
                    errors = []
                    append = errors.append
                    justname, ext = os.path.splitext(name)
                    filename = os.path.join(root, name)
                    ext = ext.replace('.','')
                    checkers = self.checkers.get(ext)
                    if checkers:
                        # read file once, pass the content to checkers
                        mode = 'r'
                        encoding = 'utf-8'
                        if ext in ['jpg', 'jpeg', 'png']:
                            encoding = None
                            mode = 'rb'
                        with fOpen(filename, mode, encoding=encoding) as f:
                            content = f.read()
                            if isinstance(content, string_types) and \
                                'p01.checker.ignore' in content:
                                # ignore this file
                                continue

                        lines = content.splitlines()
                        for checker in list(checkers.values()):
                            if not name in skipCheckerFileNames.get(ext, []):
                                for data in checker.check(basename, filename,
                                    content, lines):
                                    append(data)

                    # report found errors per file
                    self.summary(basename, filename, errors)




from __future__ import absolute_import
from builtins import object
class Foo(object):
    """A sample class forcing pyflake troubles"""

    def pdb_import(self):
        import pdb; pdb.set_trace()
        return True

    def pdb_import2(self):
        import pdb;
        pdb.set_trace()
        return True

    def pdb_import_non_reporting(self):
        import pdb;
        pdb.set_trace() # p01.checker.silence
        return True

    def dbgpclient_import(self):
        from pub.dbgpclient import brk
        brk('127.0.0.1')
        return True

    def undefined_name(self):
        foo = False
        return bar

    def non_ascii(self):
        return u'ö'



##############################################################################
#
# Copyright (c) 2010 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: tests.py 5575 2025-08-11 11:23:45Z rodrigo.ristow $
"""
from __future__ import absolute_import

import doctest
import unittest


def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('checker.txt', encoding='utf8',
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS),
        ))


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH.
# All Rights Reserved.
#
##############################################################################
"""setup

"""
from __future__ import absolute_import
__docformat__ = "reStructuredText"

import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


setup (
    name='p01.checker',
    version='0.5.16',
    author = "Adam Groszer, Roger Ineichen, Projekt01 GmbH",
    author_email = "dev@projekt01.ch",
    description = "Python, ZCML, PT, HTML, JS, CSS source checking/linting",
    long_description=(
        read('README.txt')
        + '\n\n' +
        read('CHANGES.txt')
        ),
    license = "ZPL 2.1",
    keywords = "Zope3 z3c p01 checker pyflakes lint",
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope3'],
    url = 'http://pypi.python.org/pypi/p01.checker',
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['p01'],
    extras_require = dict(
        test = [
            'zope.testing',
            ],
        cssutils = [
            'cssutils',
            ],
        ),
    install_requires = [
        'setuptools',
        'polib',
        'pyflakes',
        'lxml',
        'six',
        ],
    zip_safe = False,
    )



##############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH.
# All Rights Reserved.
#
##############################################################################
"""pyflakes (by agroszer)

$Id: pyflakes.py 5622 2025-08-26 11:37:23Z roger.ineichen $
"""

# this is a !!MODIFIED!!! version of the original
# this one returns the warnings instead of printing them to stdout

from __future__ import absolute_import
import _ast
import sys

"""
Implementation of the command-line I{pyflakes} tool.
"""

checker = __import__('pyflakes.checker').checker


def checkPYFlakes(codeString, filename):
    """Check python source code with pyflakes"""
    # first check syntax errors
    try:
        try:
            compile(codeString, filename, "exec")
        except MemoryError:
            # Python 2.4 will raise MemoryError if the source can't be
            # decoded.
            if sys.version_info[:2] == (2, 4):
                raise SyntaxError(None)
            raise
    except (SyntaxError, IndentationError) as value:
        msg = value.args[0]
        (lineno, offset, text) = value.lineno, value.offset, value.text
        # If there's an encoding problem with the file, the text is None.
        if text is None:
            # Avoid using msg, since for the only known case, it contains a
            # bogus message that claims the encoding the file declared was
            # unknown.
            return  "%s: problem decoding source" % filename
        else:
            line = text.splitlines()[-1]
            if offset is not None:
                offset = offset - (len(text) - len(line))
            result = '%s:%d: %s\n%s' % (filename, lineno, msg, line)
            if offset is not None:
                result += '\n'+" " * offset+"^"
        return result
    else:
        # Okay, it's syntactically valid. Now parse it into an ast and check it.
        tree = compile(codeString, filename, "exec", _ast.PyCF_ONLY_AST)
        w = checker.Checker(tree, filename)
        # w.messages.sort(lambda a, b: cmp(a.lineno, b.lineno))
        w.messages.sort(key=lambda x: x.lineno)
        return w.messages
